﻿using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Design;

namespace DreamsCaster;

public class AppDbContextFactory : IDesignTimeDbContextFactory<AppDbContext> {
	public AppDbContext CreateDbContext (ILoggerFactory? loggerFactory, IConfiguration config) {
		var connStr = config.GetConnectionString ("Database");

		var serverVersion = ServerVersion.AutoDetect (connStr);

		var optionsBuilder = new DbContextOptionsBuilder<AppDbContext> ()
#if VERBOSE
			.UseLoggerFactory (loggerFactory)
#endif
			.UseMySql (
				connStr,
				serverVersion,
				x => {
					x.EnableRetryOnFailure ();
				});

		return new AppDbContext (optionsBuilder.Options);
	}

	public AppDbContext CreateDbContext (string[] args = null) {
		var config = new ConfigurationBuilder ()
			.SetBasePath (AppDomain.CurrentDomain.BaseDirectory)
			.AddJsonFile ("appsettings.json")
			.Build ();

		return CreateDbContext (null, config);
	}
}