using DreamsCaster.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Console = System.Console;

namespace DreamsCaster.Controllers;

[ApiController]
[Route("[controller]")]
public class ConsoleController : ControllerBase
{
	public AppDbContext Database { get; set; }

	public ConsoleController(AppDbContext dbContext)
	{
		Database = dbContext;
	}

	[HttpGet("{consoleId:guid}")]
	public IActionResult Get([FromRoute] Guid consoleId)
	{
		if (!Database.Consoles.TryGet(consoleId, out var game)) return NotFound();
		
		return Ok(game);
	}

	[HttpDelete("{consoleId:guid}")]
	//[Authorize]
	public async Task<IActionResult> DeleteAsync([FromRoute] Guid consoleId)
	{
		if (!Database.Games.TryGet(consoleId, out var game)) return NotFound();

		foreach (var release in game.Releases)
		{
			Database.Remove(release);
		}

		await Database.SaveChangesAsync();

		Database.Games.Remove(game);
		
		await Database.SaveChangesAsync();

		return Ok();
	}

	[HttpPatch("{consoleId:guid}")]
	//[Authorize]
	public async Task<IActionResult> PatchAsync([FromRoute] Guid consoleId, [FromBody] PatchInfo newData)
	{
		if (!Database.Games.TryGet(consoleId, out Game game)) return NotFound();

		if (newData.Name.HasValue) game.Name = newData.Name;
		if (newData.Description.HasValue) game.Data.Description = newData.Description;
		if (newData.CoverPhoto.HasValue) game.Data.CoverPhoto = newData.CoverPhoto;
		
		await Database.SaveChangesAsync();

		return Ok(game);
	}

	[HttpPost]
	//[Authorize]
	public async Task<IActionResult> PostAsync([FromBody] PostConsole newConsole)
	{
		if (!Database.Manufacturers.TryGet(newConsole.ManufacturerId, out var manufacturer)) return NotFound();
		try
		{
			var game = Database.Consoles.Add(new Models.Console()
			{
				Name = newConsole.Name,
				Data = new MetaData<Models.Console>()
				{
					CoverPhoto = newConsole.CoverPhoto,
					Description = newConsole.Description
				},
				Manufacturer = manufacturer
			});

			await Database.SaveChangesAsync();
			
			return Ok(game.Entity);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
}