using DreamsCaster.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Console = System.Console;

namespace DreamsCaster.Controllers;

[ApiController]
[Route("[controller]")]
public class DeveloperController : ControllerBase
{
	public AppDbContext Database { get; set; }

	public DeveloperController(AppDbContext dbContext)
	{
		Database = dbContext;
	}

	[HttpGet("{developerId:guid}")]
	public async Task<IActionResult> GetDeveloperAsync([FromRoute] Guid developerId)
	{
		try
		{
			if (developerId == null)
			{
				return Ok();
			}

			if (!Database.Developers.Include(x=>x.Data).TryGet(developerId, out var val)) return NotFound();

			return Ok(val);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpPost]
	//[Authorize]
	public async Task<IActionResult> PostDeveloperAsync([FromBody] PostBase publisher)
	{
		try
		{
			var pub = Database.Developers.Add(new Developer()
			{
				Name = publisher.Name,
				Data = new MetaData<Developer>()
				{
					Description = publisher.Description,
					CoverPhoto = publisher.CoverPhoto
				}
			});

			await Database.SaveChangesAsync();

			return Ok(pub.Entity);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
	
	[HttpPut("{developerId:guid}")]
	//[Authorize]
	public async Task<IActionResult> PatchDeveloperAsync([FromRoute] Guid developerId, [FromBody] PatchInfo patchData)
	{
		if (!Database.Developers.TryGet(developerId, out var pub)) return NotFound();

		try
		{
			if (patchData.Name.HasValue) pub.Name = patchData.Name;
			if (patchData.Description.HasValue) pub.Data.Description = patchData.Description;
			if (patchData.CoverPhoto.HasValue) pub.Data.CoverPhoto = patchData.CoverPhoto;

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpDelete("{developerId:guid}")]
	//[Authorize]
	public async Task<IActionResult> DeleteDeveloperAsync([FromRoute] Guid developerId)
	{
		if (!Database.Developers.TryGet(developerId, out var dev )) return NotFound();

		try
		{
			foreach (var game in Database.Games.Where(x=>x.Developers.Contains(dev)))
			{
				game.Developers.Remove(dev);
			}

			Database.Developers.Remove(dev);
			await Database.SaveChangesAsync();
			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
}