using DreamsCaster.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Console = System.Console;

namespace DreamsCaster.Controllers;

[ApiController]
[Route("[controller]")]
public class ManufacturerController : ControllerBase
{
	public AppDbContext Database { get; set; }

	public ManufacturerController(AppDbContext dbContext)
	{
		Database = dbContext;
	}

	[HttpGet("{manufacturerId:guid}")]
	public async Task<IActionResult> GetManufacturerAsync([FromRoute] Guid manufacturerId)
	{
		try
		{
			if (manufacturerId == null)
			{
				return Ok();
			}

			if (!Database.Manufacturers.TryGet(manufacturerId, out var val)) return NotFound();

			return Ok(val);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpPost]
	//[Authorize]
	public async Task<IActionResult> PostManufacturerAsync([FromBody] PostBase manufacturer)
	{
		try
		{
			var pub = Database.Manufacturers.Add(new Manufacturer()
			{
				Name = manufacturer.Name,
				Data = new MetaData<Manufacturer>()
				{
					Description = manufacturer.Description,
					CoverPhoto = manufacturer.CoverPhoto
				}
			});

			await Database.SaveChangesAsync();

			return Ok(pub.Entity);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
	
	[HttpPut("{manufacturerId:guid}")]
	//[Authorize]
	public async Task<IActionResult> PatchManufacturerAsync([FromRoute] Guid manufacturerId, [FromBody] PatchInfo patchData)
	{
		if (!Database.Manufacturers.TryGet(manufacturerId, out var manufacturer)) return NotFound();

		try
		{
			if (patchData.Name.HasValue) manufacturer.Name = patchData.Name;
			if (patchData.Description.HasValue) manufacturer.Data.Description = patchData.Description;
			if (patchData.CoverPhoto.HasValue) manufacturer.Data.CoverPhoto = patchData.CoverPhoto;

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpDelete("{manufacturerId:guid}")]
	//[Authorize]
	public async Task<IActionResult> DeleteManufacturerAsync([FromRoute] Guid manufacturerId)
	{
		if (!Database.Manufacturers.TryGet(manufacturerId, out var manufacturer)) return NotFound();

		try
		{
			Database.Manufacturers.Remove(manufacturer);
			await Database.SaveChangesAsync();
			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
}