using DreamsCaster.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Console = System.Console;

namespace DreamsCaster.Controllers;

[ApiController]
[Route("[controller]")]
public class PublisherController : ControllerBase
{
	public AppDbContext Database { get; set; }

	public PublisherController(AppDbContext dbContext)
	{
		Database = dbContext;
	}

	[HttpGet("{publisherId:guid}")]
	public async Task<IActionResult> GetPublisherAsync([FromRoute] Guid publisherId)
	{
		try
		{
			if (publisherId == null)
			{
				return Ok();
			}

			if (!Database.Publishers.TryGet(publisherId, out var val)) return NotFound();

			return Ok(val);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpPost]
	//[Authorize]
	public async Task<IActionResult> PostPublisherAsync([FromBody] PostBase publisher)
	{
		try
		{
			var pub = Database.Publishers.Add(new Publisher()
			{
				Name = publisher.Name,
				Data = new MetaData<Publisher>()
				{
					Description = publisher.Description,
					CoverPhoto = publisher.CoverPhoto
				}
			});

			await Database.SaveChangesAsync();

			return Ok(pub.Entity);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
	
	[HttpPut("{publisherId:guid}")]
	//[Authorize]
	public async Task<IActionResult> PatchPublisherAsync([FromRoute] Guid publisherId, [FromBody] PatchInfo patchData)
	{
		if (!Database.Publishers.TryGet(publisherId, out var pub)) return NotFound();

		try
		{
			if (patchData.Name.HasValue) pub.Name = patchData.Name;
			if (patchData.Description.HasValue) pub.Data.Description = patchData.Description;
			if (patchData.CoverPhoto.HasValue) pub.Data.CoverPhoto = patchData.CoverPhoto;

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpDelete("{publisherId:guid}")]
	//[Authorize]
	public async Task<IActionResult> DeletePublisherAsync([FromRoute] Guid publisherId)
	{
		if (!Database.Publishers.TryGet(publisherId, out Publisher publisher)) return NotFound();

		try
		{
			Database.Publishers.Remove(publisher);
			await Database.SaveChangesAsync();
			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
}