using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using DreamsCaster.Helpers;

namespace DreamsCaster.Models;

public class IdEntity
{
	[DatabaseGenerated(DatabaseGeneratedOption.Identity)]
	[Key]
	public Guid Id { get; set; }
}

public class MetaData<T> : IdEntity
{
	public string? Description { get; set; }
	public string? CoverPhoto { get; set; }

	[Required]
	public virtual Guid OwnerId { get; set; }

	[Required]
	public virtual T Owner { get; set; }
}

public class MetaDataContainer<T> : IdEntity
{
	public string Name { get; set; }

	public virtual MetaData<T> Data { get; set; }
}


[System.Serializable]
public class PatchInfo
{
	public Optional<string> Name { get; set; } = new();
	public Optional<string> Description { get; set; } = new();
	public Optional<string> CoverPhoto { get; set; } = new();
}

[System.Serializable]
public class PostBase
{
	[Required] public string Name { get; set; }
	public string? Description { get; set; } = null;
	public string? CoverPhoto { get; set; } = null;
}

[System.Serializable]
public class PostRelease : PostBase
{
	public DateTime Release { get; set; }
	public Guid ConsoleId { get; set; }
}

[System.Serializable]	
public class PostGame : PostBase
{
	public List<PostRelease> Releases { get; set; }
}

[System.Serializable]
public class PostConsole : PostBase
{
	public Guid ManufacturerId { get; set; }
}